//
//    Copyright 2005, Thomas C. McDermott, N5EG
//    This file is part of SIGGEN - the Signal Generator program.
//
//    SIGGEN is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    SIGGEN is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SIGGEN, if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
#pragma once

//	Class to hold reference frequency and reference multiplier values
//  and to translate frequency to 48-bit DDS accumulator value

#include "stdafx.h"

#using <mscorlib.dll>

using namespace System;


public ref class FrequencySelector
{
private:
	int RefMult;			// Reference Multiplier ..  4x to 20x
	double RefFreq;			// Reference Frequency
	double FClk;			// DDS internal clock rate ( must be =< 300 MHz.)
	bool InternalRef;		// true = Internal Reference being used
	int InternalRefFreqError;	// frequency error of internal reference crystal
								// in parts per 100 million

//	FrequencySelector();	// Use default constructor	
//	~FrequencySelector();	// Use default destructor

public:
	long long unsigned FreqToDDS(double);	// Convert frequency to 48-bit DDS divisor
	long long unsigned FreqToDDSCorr(double);	// Convert frequency to 48-bit DDS divisor with correction
	double DDSToFreq(long long unsigned);	// Convert DDS divisor to frequency
	void Save(System::String ^);			// Save values to file
	void Load(System::String ^);			// Load values from file

	 property int refMult
	 {
		 int get() {return RefMult;}
		 void set(int value)
		 {
			 RefMult = value;
			 FClk = RefFreq * (double)RefMult;
		 }
	 }
	 property double refFreq
	 {
		 double get() {return RefFreq;}
		 void set(double value)
		 {
			 RefFreq = value;
			 FClk = RefFreq * (double)RefMult;
		 }
	 }
	 property bool internalRef
	 {
		 bool get() {return InternalRef;}
		 void set(bool value)
		 {
			 InternalRef = value;
			 if (InternalRef)
			 {
				 RefMult = 12;
				 RefFreq = 24000000.0;
				 FClk = RefFreq * (double)RefMult;
			 }
		 }
	 }
	 property double fClk
	 {
		 double get() {return FClk;}
	 }
	 property int internalRefFreqError
	 {
		 int get() {return InternalRefFreqError;}
		 void set(int value) {InternalRefFreqError = value;}
	 }


	//__property int get_refMult();		
	//__property double get_refFreq();
	//__property bool get_internalRef();
	//__property double get_fClk();
	//__property void set_refMult(int);
	//__property void set_refFreq(double);
	//__property void set_internalRef(bool);
	//__property void set_internalRefFreqError(int);
	//__property int get_internalRefFreqError();		
};

// Convert dB TX level to 12-bit unsigned int needed by VNA
unsigned short TxLevLinear(float dBLevel);
float dBFromLinear(int DAClevel);

float IQtoDegrees(int I, int Q);		// convert I/Q ADC reading to degrees

